/*!
@file           php_maxdb.h
@author         D030044
@ingroup        PHP MaxDB
@brief          PHP MaxDB - PHP driver for the MaxDB database
@see            

\if EMIT_LICENCE

    ========== licence begin  PHP_STANDARD
    Copyright (c) 2001-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of either the Artistic License, as
    specified in the Perl README file or the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end


\endif
*/

#ifndef PHP_MAXDB_H
#define PHP_MAXDB_H

extern zend_module_entry maxdb_module_entry;
#define phpext_maxdb_ptr &maxdb_module_entry

#ifdef PHP_WIN32
#define PHP_MAXDB_API __declspec(dllexport)
#else
#define PHP_MAXDB_API
#endif

#ifdef ZTS
#include "TSRM.h"
#endif

#include <limits.h>
#include <stdlib.h>
#include "SQLDBC_C.h"

#define MAXDB_REPORT_OFF      0  /* no error reporting */
#define MAXDB_REPORT_ERROR    2  /* error reporting on */

PHP_MINIT_FUNCTION(maxdb);
PHP_MSHUTDOWN_FUNCTION(maxdb);
PHP_RINIT_FUNCTION(maxdb);
PHP_RSHUTDOWN_FUNCTION(maxdb);
PHP_MINFO_FUNCTION(maxdb);

PHP_FUNCTION(maxdb_debug);
PHP_FUNCTION(maxdb_enable_reads_from_master);
PHP_FUNCTION(maxdb_disable_reads_from_master);
PHP_FUNCTION(maxdb_enable_rpl_parse);
PHP_FUNCTION(maxdb_disable_rpl_parse);
PHP_FUNCTION(maxdb_rpl_parse_enabled);
PHP_FUNCTION(maxdb_rpl_probe);
PHP_FUNCTION(maxdb_rpl_query_type);
PHP_FUNCTION(maxdb_dump_debug_info);
PHP_FUNCTION(maxdb_embedded_connect);
PHP_FUNCTION(maxdb_select_db);
PHP_FUNCTION(maxdb_stat);
PHP_FUNCTION(maxdb_thread_id);
PHP_FUNCTION(maxdb_get_proto_info);
PHP_FUNCTION(maxdb_get_server_info);
PHP_FUNCTION(maxdb_server_init);
PHP_FUNCTION(maxdb_server_end);
PHP_FUNCTION(maxdb_get_server_version);
PHP_FUNCTION(maxdb_get_client_info);
PHP_FUNCTION(maxdb_get_client_version);
PHP_FUNCTION(maxdb_get_host_info);
PHP_FUNCTION(maxdb_autocommit);
PHP_FUNCTION(maxdb_report);
PHP_FUNCTION(maxdb_options);
PHP_FUNCTION(maxdb_init);
PHP_FUNCTION(maxdb_real_connect);
PHP_FUNCTION(maxdb_ssl_set);
PHP_FUNCTION(maxdb_connect);
PHP_FUNCTION(maxdb_connect_errno);
PHP_FUNCTION(maxdb_connect_error);
PHP_FUNCTION(maxdb_rollback);
PHP_FUNCTION(maxdb_kill);
PHP_FUNCTION(maxdb_change_user);
PHP_FUNCTION(maxdb_insert_id);
PHP_FUNCTION(maxdb_real_escape_string);
PHP_FUNCTION(maxdb_query);
PHP_FUNCTION(maxdb_send_query);
PHP_FUNCTION(maxdb_real_query);
PHP_FUNCTION(maxdb_master_query);
PHP_FUNCTION(maxdb_multi_query);
PHP_FUNCTION(maxdb_store_result);
PHP_FUNCTION(maxdb_use_result);
PHP_FUNCTION(maxdb_next_result);
PHP_FUNCTION(maxdb_more_results);
PHP_FUNCTION(maxdb_sqlstate);
PHP_FUNCTION(maxdb_prepare);
PHP_FUNCTION(maxdb_stmt_init);
PHP_FUNCTION(maxdb_stmt_prepare);
PHP_FUNCTION(maxdb_stmt_execute);
PHP_FUNCTION(maxdb_stmt_errno);
PHP_FUNCTION(maxdb_stmt_error);
PHP_FUNCTION(maxdb_stmt_affected_rows);
PHP_FUNCTION(maxdb_stmt_sqlstate);
PHP_FUNCTION(maxdb_stmt_param_count);
PHP_FUNCTION(maxdb_stmt_num_rows);
PHP_FUNCTION(maxdb_stmt_bind_param);
PHP_FUNCTION(maxdb_stmt_bind_result);
PHP_FUNCTION(maxdb_stmt_fetch);
PHP_FUNCTION(maxdb_stmt_data_seek);
PHP_FUNCTION(maxdb_stmt_send_long_data);
PHP_FUNCTION(maxdb_stmt_close_long_data);
PHP_FUNCTION(maxdb_stmt_store_result);
PHP_FUNCTION(maxdb_stmt_free_result);
PHP_FUNCTION(maxdb_stmt_reset);
PHP_FUNCTION(maxdb_stmt_close);
PHP_FUNCTION(maxdb_stmt_result_metadata);
PHP_FUNCTION(maxdb_free_result);
PHP_FUNCTION(maxdb_data_seek);
PHP_FUNCTION(maxdb_fetch_row);
PHP_FUNCTION(maxdb_fetch_assoc);
PHP_FUNCTION(maxdb_fetch_array);
PHP_FUNCTION(maxdb_fetch_object);
PHP_FUNCTION(maxdb_num_fields);
PHP_FUNCTION(maxdb_num_rows);
PHP_FUNCTION(maxdb_fetch_field);
PHP_FUNCTION(maxdb_fetch_fields);
PHP_FUNCTION(maxdb_fetch_field_direct);
PHP_FUNCTION(maxdb_fetch_lengths);
PHP_FUNCTION(maxdb_field_count);
PHP_FUNCTION(maxdb_field_seek);
PHP_FUNCTION(maxdb_field_tell);
PHP_FUNCTION(maxdb_affected_rows);
PHP_FUNCTION(maxdb_info);
PHP_FUNCTION(maxdb_character_set_name);
PHP_FUNCTION(maxdb_ping);
PHP_FUNCTION(maxdb_error);
PHP_FUNCTION(maxdb_errno);
PHP_FUNCTION(maxdb_warning_count);
PHP_FUNCTION(maxdb_close);
PHP_FUNCTION(maxdb_commit);

/*
 * internal structure for holding connection
 */
typedef struct maxdb_connection {
	char                     *m_host;            /* hostname used for connect */
	char                     *m_db;              /* database name used for connect */
	SQLDBC_ConnectProperties *m_connprop;        /* connect properties for connect */
	SQLDBC_Connection        *m_connection;      /* SQLDBC connection */
	SQLDBC_ErrorHndl         *m_errhndl;         /* error handle for connect errors */
	SQLDBC_Statement         *m_stmt;            /* statement for maxdb_query() */
	SQLDBC_PreparedStatement *m_prepstmt;        /* statement for prepare */
	SQLDBC_Bool               m_isprepared;      /* flag if m_prepstmt was prepared or not */
	SQLDBC_Int4               m_rowsaffected;    /* rows affected in last database query */
	SQLDBC_Int4               m_numcols;         /* number of result columns (for maxdb_field_count()) */
	SQLDBC_Int4               m_id;              /* resource identifier */
	SQLDBC_Int4               m_resultid;        /* resource identifier for resultset of m_stmt */
} maxdb_connection;

/*
 * internal structure for holding column of a result set
 */
typedef struct maxdb_result_value {
	char           *buf;                         /* pointer to internal buffer */
	char            bufName[64];                 /* name of the buffer */
    SQLDBC_Length   bufLen;                      /* maximum length of buffer within the result */
    SQLDBC_HostType hostType;                    /* column datatyp */
    SQLDBC_Length   indicator;                   /* column indicator */
    SQLDBC_Bool     chopBlanks;                  /* flag if blanks chopped from string */
	SQLDBC_Bool     isint;                       /* flag if value is int bound as ASCII */
} maxdb_result_value;

/*
 * internal structure for holding a result set
 */
typedef struct maxdb_result {
	SQLDBC_Int4               m_id;              /* resource identifier */
	SQLDBC_ResultSet         *m_resultset;       /* SQLDBC result set */
	SQLDBC_ResultSetMetaData *m_rsmd;            /* result set metadata */
	maxdb_result_value       *m_cols;            /* columns of the result set */
	SQLDBC_UInt4              m_numcols;         /* number of columns in result set */
	SQLDBC_Bool               m_rowNotFound;     /* flag if rows found or not */
	SQLDBC_Int4               m_rowsaffected;	 /* rows affected in the last database query */
	SQLDBC_UInt4              m_rowSetSize;      /* size of the used rowset */
	SQLDBC_Int4               m_lastindex;       /* last column index (for maxdb_fetch_field()) */
	zval                    **m_vars;            /* PHP vars bound to result set */
	SQLDBC_UInt4              m_varscnt;         /* number of PHP vars bound to result set */
	SQLDBC_Bool               m_noclose;         /* flag if result set should be closed or not */
	SQLDBC_Int4               m_connid;          /* resource identifier for connection */
} maxdb_result;

/*
 * internal structure for holding a statement parameter
 */
typedef struct maxdb_bind_param {
    char           *buf;                         /* pointer to internal buffer */
	char           *lob;                         /* buffer holding temporary lob data */
	SQLDBC_Int4     paramlen;                    /* length of the bound parameter */
	int             sqltype;                     /* internal sqltype of the bound parameter */
	SQLDBC_HostType hostType;                    /* hosttype of the bound parameter */
	SQLDBC_Length   indicator;                   /* indicator of the bound parameter */
	zval           *var;                         /* PHP var of the parameter */
} maxdb_bind_param;

/*
 * internal structure for holding a prepared statement
 */
typedef struct maxdb_prepstmt {
	SQLDBC_Int4               m_id;              /* resource identifier */
	SQLDBC_PreparedStatement *m_prepstmt;        /* SQLDBC prepared statement */
    maxdb_bind_param         *m_bindparams;      /* prepared statement parameters */
	SQLDBC_Int2               m_parcnt;          /* number of prepared statement parameters */
	SQLDBC_ParameterMetaData *m_parammetadata;   /* parameter metadata */
	SQLDBC_Bool               m_rownotfound;     /* flag if row not found or not */
	maxdb_result             *m_result;          /* result set for prepared statement (if query) */
	SQLDBC_Bool               m_isexecuted;      /* flag if statement was already executed or not */
	zval                     *m_clink;           /* link to connection */
} maxdb_prepstmt;

/* 
  	Declare any global variables you may need between the BEGIN
	and END macros here:     */

ZEND_BEGIN_MODULE_GLOBALS(maxdb)
	char                     *default_host;      /* default hostname for connect */
	char                     *default_db;        /* default databasename for connect */   
	char                     *default_user;      /* default username for connect */
	char                     *default_pw;        /* default password for connect */
	char                     *db;                /* current database used in connection */
	char                     *user;              /* current user used in connection */
	char                     *password;          /* current password used in connection */
	SQLDBC_Environment       *m_maxDBEnv;        /* SQLDBC environment */
	int                       report_mode;       /* error reporting mode */
	SQLDBC_UInt4              long_readlen;      /* max. length of long data to read */
	int                       error_no;          /* error number from last query */
	char                     *error_msg;         /* error message from last query */
	SQLDBC_ConnectProperties *conn_prop;         /* SQLDBC connect properties */
	int                       ssl_error;         /* whether ssl errors should be reported */
	SQLDBC_UInt4              thread_id;         /* thread id count for connections */
ZEND_END_MODULE_GLOBALS(maxdb)


/* In every utility function you add that needs to use variables 
   in php_maxdb_globals, call TSRMLS_FETCH(); after declaring other 
   variables used by that function, or better yet, pass in TSRMLS_CC
   after the last function argument and declare your utility function
   with TSRMLS_DC after the last declared argument.  Always refer to
   the globals in your function as MAXDB_G(variable).  You are 
   encouraged to rename these macros something shorter, see
   examples in any other php module directory.
*/

#ifdef ZTS
#define MAXDB_G(v) TSRMG(maxdb_globals_id, zend_maxdb_globals *, v)
#else
#define MAXDB_G(v) (maxdb_globals.v)
#endif

#endif	/* PHP_MAXDB_H */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
