#/*!
#\@file           genSQLDBCdoc.pl
#\@author         DL SAP DB INTERFACES
#\@ingroup        SQLDBC
#\@brief          SAP Database Connectivity
#\@see            
#
#       This script patches the generated doxygen documentation 
#       for the c++ interface SQLDBC
#
#\\if EMIT_LICENCE
#
#	========== licence begin  GPL
#	Copyright (c) 2001-2004 SAP AG
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#	========== licence end
#
#
#\\endif
#*/

#use strict;
use Env;
use File::Path;
use Getopt::Long;
use Getopt::Std;

GetOptions("dir=s", "with-floss", "help") || usage();

if (!getopts ('h') || $opt_help || !$opt_dir) {
    usage();
}

my $infile = "$opt_dir/tree.js";
my $outfile = "$$opt_dir/tree.js.tmp";
open(FILE_IN, "<$infile")  || die "Can't open '$infile' (input): $!";
open(FILE_OUT, ">$outfile")  || die "Can't open '$outfile' (output): $!";
my $firstline=1;
while(<FILE_IN>) {
#
#   Skip if already modified
#
    if ($_=~/\/\/ tree.js modified by genSQLDBCdoc/) {
        exit 0;
    }
    if ($firstline) {
        print  FILE_OUT "// tree.js modified by genSQLDBCdoc\n";
        $firstline=0;
    }
#
#   Insert link for Native C Reference
#
    if ($_=~/\s+insDoc\(.*\"SQLDBC_C.h\".*\"(SQLDBC__C.*)\".*/){
        $sqldbc_c_ref = $1;
    }
    if ($_=~/\s+insDoc\(.*\"Compound Members\".*/){
        print  FILE_OUT "insDoc(foldersTree, gLnk(\"Native C Reference\", \"\", \"$sqldbc_c_ref\"))\n";
    }	      
#
#   Insert Installation chapter
#
    if ($_=~/\s+insDoc\(.*\"SQL Database Connectivity.*\".*/){
        print FILE_OUT $_;
        print FILE_OUT "insDoc(foldersTree, gLnk(\"Installation\", \"\", \"install.html\"))\n";
        next;
    }	          
    print FILE_OUT $_;
}

#
#   Append Examples chapter
#
printExamplesChapter(\*FILE_OUT);

#
#   Append License chapter
#
printLicenseChapter(\*FILE_OUT);

close(FILE_OUT); close(FILE_IN);

unlink "$infile";
rename "$outfile", "$infile";
exit 0;

sub printLicenseChapter {
    my $fh = shift;
    print $fh "aux1 = insFld(foldersTree, gFld(\"License\", \"\", \"license.html\"))\n";
    if ($opt_with_floss) {
        print $fh "insDoc(aux1, gLnk(\"MaxDB FLOSS license exception\", \"\", \"FLOSS.html\"))\n";
    }
}

sub printExamplesChapter {
    my $fh = shift;
    print $fh "aux1 = insFld(foldersTree, gFld(\"Examples\", \"\", \"examples.html\"))\n";
}

sub usage() {
    print <DATA>;
    exit 1;
}

__DATA__

 USAGE: genSQLDBCdoc [-h] --dir=<path> [options]

           Patches the doxygen generated SQLDBC documnetation.

Options:  --dir=<path>    Path to the html doc tree
          --help          This help
 
          --with-floss    Includes FLOSS license exception
